/*
 * balloon.h
 *
 * Ballon help window.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: balloon.h,v $
 * Revision 1.7  1999/03/10 03:49:50  robertj
 * More documentation adjustments.
 *
 * Revision 1.6  1999/03/09 08:01:47  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.5  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.4  1998/09/23 06:22:51  robertj
 * Added open source copyright license.
 *
 * Revision 1.3  1995/07/31 12:05:11  robertj
 * Split off pop up window from balloon.
 *
 * Revision 1.2  1995/06/17 11:12:19  robertj
 * Documentation update.
 *
 * Revision 1.1  1995/04/01 08:24:14  robertj
 * Initial revision
 *
 */


#ifndef _PBALLOON
#define _PBALLOON

#ifdef __GNUC__
#pragma interface
#endif


/**This class defines a user interface entity for supporting balloon help.
 */
class PBalloon : public PPopUp
{
  PCLASSINFO(PBalloon, PPopUp);

  public:
   /**Create a new balloon help interactor.  */
    PBalloon(
      PInteractor * parent  /// Interactor that owns the dialog.
    );
   /**Create a new balloon help interactor. The first form does not display a
       balloon help window and is mainly used as an ancestor to some special
       version of the window. The second form immediately shows the window in
       the default position relative to the parent specified.
     */
    PBalloon(
      PInteractor * parent,  /// Interactor that owns the dialog.
      const PString & text   /// Resource identifier for loading controls.
    );


  /**@name New functions for class */
   /**Get the current balloon help text.
    
       @return
       string for the balloon help.
     */
    const PString & GetText() const;

   /**Set the current balloon help text.
     */
    void SetText(
      const PString & text   /// New balloon help text.
    );


  protected:
    // System callback functions.
   /**The system calls this whenever it requires that the interactors usable
       area needs to be updated and redrawn.

       The canvas is usually a \Ref{PRedrawCanvas} instance with its clip
       region set to the current update region as determined by the
       \Ref{PInteractor::Invalidate()} and \Ref{PInteractor::Validate()}
       functions as well as those areas that were obscured by overlapping
       windows.
     */
    virtual void OnRedraw(
      PCanvas & canvas   /// Canvas to use when drawing the interactor contents.
    );


    // Member variables
    /** Text contained in the balloon help window. */
    PString balloonText;


  private:
    void Construct();
      // Common construction code.
};


#endif /// _PBALLOON


// End Of File ///////////////////////////////////////////////////////////////
