/*
 * checkbox.h
 *
 * Check box control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: checkbox.h,v $
 * Revision 1.19  2001/05/22 12:49:32  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.18  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.17  1999/03/09 08:01:47  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.16  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.15  1998/09/23 06:23:01  robertj
 * Added open source copyright license.
 *
 * Revision 1.14  1995/03/14 12:41:05  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.13  1995/01/27  11:04:15  robertj
 * Changed state of checkbox from CheckValue to BOOL.
 *
 * Revision 1.12  1994/11/28  12:31:46  robertj
 * Documentation.
 *
 * Revision 1.11  1994/10/30  11:46:30  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.10  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.9  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.8  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.7  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.6  1994/01/15  02:50:29  robertj
 * Moved control contructors to common and added a platform dependent Construct() function.
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PCHECKBOX

#ifdef __GNUC__
#pragma interface
#endif



/**A 2 state check box control that has {\it Checked}, {\it Unchecked}
   states.

   The exact appearence of a check box is platform dependent, however, they
   all have some visual distinction between the two states plus a string
   label associated with it.
 */
class PCheckBox : public PCheck3WayBox
{
  PCLASSINFO(PCheckBox, PCheck3WayBox);

  public:
   /**Create a check box control with the specified name next to it,
       callback function and the initial check state.
     */
    PCheckBox(
      PInteractor * parent,     /// Interactor into which the control is placed.
      BOOL value = FALSE        /// Initial state for the check box.
    );
    PCheckBox(
      PInteractor * parent,     /// Interactor into which the control is placed.
      const PString & name,     /// Name to be placed next to the check box.
      BOOL value = FALSE        /// Initial state for the check box.
    );
    PCheckBox(
      PInteractor * parent,     /// Interactor into which the control is placed.
      const PString & name,     /// Name to be placed next to the check box.
      const PNotifier & notify  /// Function to call when changes state.
    );

   /**Create control from interactor layout with the specified control ID.
       This is mainly used in support of resource based dialogs.
     */
    PCheckBox(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      BOOL * valuePtr             /// Variable to change to the check box state.
    );

    /** Destroy the check box. */
    virtual ~PCheckBox();


  /**@name Overrides from class PControl */
   /**This function transfers the value of the control to or from the variable
       pointed to by the value pointer member variable.
     */
    virtual void TransferValue(
      int option 
     /**Transfer value option. When this is -1 when the function transfers
         the value from the value pointer into the control. This is called in
         \Ref{PDialog::OnInit()} function. When option is zero then the
         function transfers the value from the control to the value pointer
         variable. This is called just before the callback function every time
         the check box state changes.
       */
    );


    /**@name Overrides from class PCheck3WayBox */
   /**Get the current state of the check box.

       @return
       check box state.
     */
    BOOL GetValue() const;

   /**Set the check state of the check box. This will be updated on the
       screen immediately (within OS contsraints).

       Note that unlike the \Ref{PCheck3WayBox} class this function is
       limited to the Checked (TRUE) and Unchecked (FALSE) values.
     */
    virtual void SetValue(
      BOOL newVal  /// New state to set the check box to.
    );

   /**Get the current value pointer associated with the control. The variable
       pointed to by this is autamatically updated with the current value of
       the check box.

       @return
       value pointer associated with the control.
     */
    BOOL * GetValuePointer() const;

   /**Set the current value pointer associated with the control. The variable
       pointed to by this is autamatically updated with the current value of
       the check box.
     */
    void SetValuePointer(
      BOOL * ptr  /// New value pointer to associate with the control.
    );


// Include platform dependent part of class
#include <pwlib/checkbox.h>
};


// End Of File ///////////////////////////////////////////////////////////////
