/*
 * dirdlg.h
 *
 * Directory selection dialog.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: dirdlg.h,v $
 * Revision 1.16  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.15  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.14  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.13  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.12  1998/09/23 06:23:23  robertj
 * Added open source copyright license.
 *
 * Revision 1.11  1995/06/17 11:12:31  robertj
 * Documentation update.
 *
 * Revision 1.10  1995/03/14 12:41:21  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.9  1994/12/21  11:52:53  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.8  1994/12/15  12:47:12  robertj
 * Documentation.
 *
 * Revision 1.7  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.6  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.5  1994/04/11  14:13:05  robertj
 * Added extra standard resource IDs and normalised dialog ids.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _POPENDIRDIALOG

#ifdef __GNUC__
#pragma interface
#endif


/**A standard dialog for obtaining a directory name. The exact appearance of
   the dialog is platform specific, however they all consists of the same
   basic functions.

   Note that unlike most dialogs, most operations such as
   \Ref{PInteractor::Show()}, \Ref{PinteractorLayout::GetControl()} etc may
   {\bf not} be executed before the \Ref{PModalDialog::RunModal()} function
   has been executed. Furthermore, they are not available {\bf after}
   \Ref{PModalDialog::RunModal()} has returned. Similarly some functions such
   as \Ref{PFileDialog::SetDirectory()} are not available {\bf within} the
   context of \Ref{PModalDialog::RunModal()}. This is due to the
   implementation on some platforms which goes to great lengths to use the
   actual standard file dialog for the platform.
 */
class POpenDirDialog : public PFileDialog
{
  PCLASSINFO(POpenDirDialog, PFileDialog);

  public:
   /**Create a new standard open directory dialog. The actual appearence is
       platform dependent and is displayed using the standard resource. This
       resource may be overridden by the #resID# parameter but must
       still contain all of the controls of the standard resource.
     */
          POpenDirDialog(
      PInteractor * parent,   /// Owner interactor for the dialog.
      PRESOURCE_ID resID = PSTD_ID_DIALOG_OPEN_DIR
        /// Resource identifier for the dialog description resource.
    );

    /** Destroy the standard open directory dialog. */
    virtual ~POpenDirDialog();


// Include platform dependent part of class
#include <pwlib/dirdlg.h>
};


// End Of File ///////////////////////////////////////////////////////////////
