/*
 * hscrollb.h
 *
 * Horizontal scroll bar control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: hscrollb.h,v $
 * Revision 1.16  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.15  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.14  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.13  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.12  1998/09/23 06:23:43  robertj
 * Added open source copyright license.
 *
 * Revision 1.11  1994/12/21 11:53:00  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.10  1994/10/30  11:46:38  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.9  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.8  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.7  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.6  1994/01/15  02:50:29  robertj
 * Moved control contructors to common and added a platform dependent Construct() function.
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PHORIZONTALSCROLLBAR

#ifdef __GNUC__
#pragma interface
#endif


/**A horizontal scroll bar control for visually setting an integer value.

   The exact appearance of a scroll bar is platform dependent. However certain
   functionality exists for all platforms. These are that the scroll bar allows
   values between a minimum and maximum to be set via small nudges (typically
   a up or down arrow), large nudges (typically the page up or down area to
   either side of the thumb) or by explicitly setting a value (typically by
   dragging the thumb).
 */
class PHorizontalScrollBar : public PScrollBar
{
  PCLASSINFO(PHorizontalScrollBar, PScrollBar);

  public:
    /** Create a new horizontal scroll bar. */
    PHorizontalScrollBar(
      PInteractor * parent,  /// Interactor into which the control is placed.
      PSCROLLBAR_VALUE max = 100, /// Maximum value for scroll bar.
      PSCROLLBAR_VALUE min = 0,   /// Minimum value for scroll bar.
      PSCROLLBAR_VALUE val = 0,   /// Initial value for scroll bar.
      PSCROLLBAR_VALUE sml = 1,   /// Amount to move with the nudge button.
      PSCROLLBAR_VALUE lge = 10   /// Amount to move with the paging area.
    );
    PHorizontalScrollBar(
      PInteractor * parent,  /// Interactor into which the control is placed.
      const PNotifier & func,     /// Function to call when changes state.
      PSCROLLBAR_VALUE max = 100, /// Maximum value for scroll bar.
      PSCROLLBAR_VALUE min = 0,   /// Minimum value for scroll bar.
      PSCROLLBAR_VALUE val = 0,   /// Initial value for scroll bar.
      PSCROLLBAR_VALUE sml = 1,   /// Amount to move with the nudge button.
      PSCROLLBAR_VALUE lge = 10   /// Amount to move with the paging area.
    );
      
    /** Create control from interactor layout with the specified control ID. */
    PHorizontalScrollBar(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      PSCROLLBAR_VALUE * valuePtr /// Variable to change to the scrollbar value.
    );


// Include platform dependent part of class
#include <pwlib/hscrollb.h>
};


// End Of File ///////////////////////////////////////////////////////////////
