/*
 * ncontrol.h
 *
 * Named Control class.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: ncontrol.h,v $
 * Revision 1.15  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.14  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.13  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.12  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.11  1998/09/23 06:24:27  robertj
 * Added open source copyright license.
 *
 * Revision 1.10  1995/03/14 12:41:52  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.9  1995/01/06  10:31:03  robertj
 * Documentation.
 *
 * Revision 1.8  1994/10/30  11:46:53  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.7  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.6  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.5  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PNAMEDCONTROL

#ifdef __GNUC__
#pragma interface
#endif


/**This class defines a user inteface entity that allows the user to control
   some attribute/data within the system.
   
   In addition to the standard PControl functionality a named control also
   has a title or name associated with it, eg a check box has a text string
   next to the graphic.
 */
class PNamedControl : public PControl
{
  PCLASSINFO(PNamedControl, PControl);

  public:
      /** Create a named control with the specified name. */
    PNamedControl(
      PInteractor * parent    /// Interactor into which the control is placed.
    );
    PNamedControl(
      PInteractor * parent,   /// Interactor into which the control is placed.
      const PString & name,   /// Name for use by the control.
      const PNotifier & func, /// Function to call when control changes state.
      void * valuePtr         /// Variable to change to the controls state.
    );

   /**Create control from interactor layout with the specified control ID.
       This is mainly used in support of resource based dialogs.
     */
    PNamedControl(
      PInteractorLayout * parent, /// Interactor on which the control is placed.
      PRESOURCE_ID ctlID,       /// Identifier for the control in the layout.
      const PNotifier & func,   /// Function to call when control changes state.
      void * valuePtr           /// Variable to change to the controls state.
    );


    /** Destroy the named control. */
    virtual ~PNamedControl();


    /**@name New functions for class */
   /**Get the name that is currently used by the control.

       @return
       string for the control name.
     */
    PString GetName() const;

    /** Set the name that is to be next to the control. */
    void SetName(
      const PString & name    /// New name for use by the control.
    );


// Include platform dependent part of class
#include <pwlib/ncontrol.h>
};


// End Of File ///////////////////////////////////////////////////////////////
