/*
 * pclrdlg.h
 *
 * Colour selection dialog.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: pclrdlg.h,v $
 * Revision 1.9  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.8  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.7  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.6  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.5  1998/09/23 06:27:47  robertj
 * Added open source copyright license.
 *
 * Revision 1.4  1995/08/12 22:29:04  robertj
 * Moved Construct() function to platform dependent code.
 *
 * Revision 1.3  1995/06/17 11:12:22  robertj
 * Documentation update.
 *
 * Revision 1.2  1995/03/14 12:41:08  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.1  1995/01/22  04:49:21  robertj
 * Initial revision
 *
 */


#define _PCOLOURDIALOG

#ifdef __GNUC__
#pragma interface
#endif


/**System specific standard dialogs for getting a colour specification.

   Note that unlike most dialogs, most operations such as
   \Ref{PInteractor::Show()}, \Ref{PInterqactorLayout::GetControl()} etc may
   {\bf not} be executed before the \Ref{PModalDialog::RunModal()} function
   has been executed. Furthermore, they are not available {\bf after}
   \Ref{PModalDialog::RunModal()} has returned. This is due to the
   implementation on some platforms which goes to great lengths to use the
   actual standard colour dialog for the platform.
 */
class PColourDialog : public PModalDialog
{
  PCLASSINFO(PColourDialog, PModalDialog);

  public:
   /**Create the standard colour dialog. The second form of constructor
       preinitialises the fields in the dialog to that specified in the
       colour specification class. Otherwise a system default will be used.
     */
    PColourDialog(
      PInteractor * parent,    /// Owner interactor for the dialog.
      PRESOURCE_ID resID = PSTD_ID_DIALOG_COLOUR
        /// Resource identifier for the dialog description resource.
    );
    PColourDialog(
      PInteractor * parent,    /// Owner interactor for the dialog.
      const PColour & colour,  /// Initial colour specification.
      PRESOURCE_ID resID = PSTD_ID_DIALOG_COLOUR
        /// Resource identifier for the dialog description resource.
    );

    /** Destroy the standard colour dialog. */
    ~PColourDialog();


  /**@name New functions for class */
   /**Set the initial colour for the dialog. This function will only have an
       effect before the \Ref{PModalDialog::RunModal()} function is called.
       After that it is ignored.
     */
    void SetColour(const PColour & col);

   /**Get the colour specification set by this dialog. Before the
       {\bf return} from the \Ref{PModalDialog::RunModal()} function, this
       will get the inital colour as set by the constructor or the
       \Ref{SetColour()} function. After the \Ref{PModalDialog::RunModal()}
       function has returned TRUE, it will get the selected colour.

       @return
       reference to \Ref{PColour} member variable in this class.
     */
    const PColour & GetColour() const;


  protected:
  // Member variables
    /** Colour specification for the dialog. */
    PColour colour;


  private:
  // New members for class
    void Construct(PRESOURCE_ID resID);
    // Constructor code common to all constructors.


// Include platform dependent part of class
#include <pwlib/pclrdlg.h>
};


// End Of File ///////////////////////////////////////////////////////////////
