/*
 * pwedbox.h
 *
 * password edit text box control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: pwedbox.h,v $
 * Revision 1.12  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.11  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.10  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.9  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.8  1998/09/23 06:28:29  robertj
 * Added open source copyright license.
 *
 * Revision 1.7  1995/06/04 12:35:21  robertj
 * Added common constructor - Unix implementation requirement.
 * Removed redundent destructor.
 *
 * Revision 1.6  1995/03/14 12:42:19  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.5  1995/01/14  06:19:23  robertj
 * Documentation
 *
 * Revision 1.4  1994/10/30  11:46:57  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.3  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.2  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.1  1994/04/11  14:15:05  robertj
 * Initial revision
 *
 */

#define _PPASSWORDEDITBOX

#ifdef __GNUC__
#pragma interface
#endif

/**This class is a text editing box that does not display characters entered.
   It is typically used for entering password in secure systems. Other than
   the display of a simgle character for all real characters entered it is
   identical to the ancestor \Ref{PEditBox}.
 */
class PPasswordEditBox : public PEditBox
{
  PCLASSINFO(PPasswordEditBox, PEditBox);

  public:
    /** Create a password edit box control. */
    PPasswordEditBox(
      PInteractor * parent  /// Interactor into which the control is placed.
    );

    /** Create control from interactor layout with the specified control ID. */
    PPasswordEditBox(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      PString * valuePtr          /// Variable to change to the editor value.
    );


  /**@name New functions for class */
   /**Set the character displayed instead of entered characters. This gives
       the user feedback on how many characters were entered but not what
       characters.
     */
    void SetPasswordCharacter(
      char c   /// New character to display.
    );

   /**Get the character displayed instead of entered characters. This gives
       the user feedback on how many characters were entered but not what
       characters.
       
       @return
       character to display.
     */
    char GetPasswordCharacter();


  private:
    void Construct();
      // Common construction code.


// Include platform dependent part of class
#include <pwlib/pwedbox.h>
};


// End Of File ///////////////////////////////////////////////////////////////
