/*
 * resdata.h
 *
 * Artbitrary resource data.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: resdata.h,v $
 * Revision 1.15  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.14  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.13  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.12  1998/09/23 06:28:50  robertj
 * Added open source copyright license.
 *
 * Revision 1.11  1995/03/14 12:42:26  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.10  1995/01/14  06:19:31  robertj
 * Documentation
 *
 * Revision 1.9  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.8  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.7  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.6  1994/04/20  12:17:44  robertj
 * *** empty log message ***
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PRESOURCEDATA

#ifdef __GNUC__
#pragma interface
#endif


/**A class representing user defined resource data object. This allows an
   application to access resources other than the standard ones. The
   interpretation of the data within the resource is up to the application.
   
   This class provides access to a simple memory block containing the data in
   the resource. This would typically be created using the DATA keyword in a
   PWRC compiled source file.
 */
class PResourceData : public PContainer
{
  PCONTAINERINFO(PResourceData, PContainer);

  public:
   /**Create a new resource object, loading the data from the applications
       resources. Care must be made not to create resources that use a standard
       resource type name eg "ICON". THis is especially reelevant to the
       MacOS platform which make extensive use of resources.
     */
    PResourceData(
      const PString & resType,    /// Resource type string of 4 characters.
      PRESOURCE_ID resID      /// Unique identifier (within type) for resource.
    );


  /**@name Overrides from class PObject */
   /**Determine if the instances are of the same resource. Note that this
       checks that they are of the same resource type and identifier and not
       merely having the same internal data.
       
       @return
       #EqualTo# if reference the same resource, otherwise
       #GreaterThan#.
     */
    virtual Comparison Compare(
      const PObject & obj   /// Other resource data object to comapre against.
    ) const;


  /**@name New memeber functions for class */
    /** Get a byte from the resource at the specified offset. */
    BYTE operator[](
      PINDEX offset     /// Offset into the resource data.
    ) const;

    /** Return a pointer to the resource data in memory. */
    operator const BYTE *() const;


  private:
    // Overrides from class PContainer
   /* Set the size of the container. As this type of container can only ever
       contain one resource, this function is ignored.
       
       @return
       TRUE always.
     */
    virtual BOOL SetSize(
      PINDEX newSize    /// New size of container.
    );


// Include platform dependent part of class
#include <pwlib/resdata.h>
};


// End Of File ///////////////////////////////////////////////////////////////
