/*
 * rstring.h
 *
 * Resource loaded string.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: rstring.h,v $
 * Revision 1.14  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.13  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.12  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.11  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.10  1998/09/23 06:28:54  robertj
 * Added open source copyright license.
 *
 * Revision 1.9  1995/03/14 12:42:29  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.8  1995/01/14  06:19:34  robertj
 * Documentation
 *
 * Revision 1.7  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.6  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.5  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PRESOURCESTRING

#ifdef __GNUC__
#pragma interface
#endif


/**A class representing a string loaded from an application resource.
 */
class PResourceString : public PString
{
  PCLASSINFO(PResourceString, PString);

  public:
   /**Create a string loading it from the application resources. If there
       is no string resource of that name, then an empty string or the
       #defaultString# parameter is returned.

       Note the platform standard string resource is used. For the MacOS this
       is actually a STR# resource in groups of 10 strings. The actual STR#
       resource ID is #resID# divided by 10.
     */
    PResourceString(
      PRESOURCE_ID resID    /// Unique identifier for string.
    );
    PResourceString(
      PRESOURCE_ID resID,   /// Unique identifier for string.
      const PString & defaultString  /// Value to use if resource not present.
    );


  private:
    void Construct(
      PRESOURCE_ID resID    /// Unique identifier for string.
    );
    // Common, platform dependent, construction code for each constructor.


// Include platform dependent part of class
#include <pwlib/rstring.h>
};


// End Of File ///////////////////////////////////////////////////////////////
