/*
 * staticon.h
 *
 * Static icon control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: staticon.h,v $
 * Revision 1.18  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.17  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.16  1999/03/09 08:01:50  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.15  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.14  1998/09/23 06:29:20  robertj
 * Added open source copyright license.
 *
 * Revision 1.13  1995/03/14 12:42:42  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.12  1995/01/16  09:42:10  robertj
 * Documentation.
 *
 * Revision 1.11  1994/10/30  11:47:19  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.10  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.9  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.8  1994/07/21  23:47:51  robertj
 * Added platform dependent Construct().
 *
 * Revision 1.7  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.6  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.5  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.4  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PSTATICICON

#ifdef __GNUC__
#pragma interface
#endif


/**This control displays a simple icon. The icon is displayed without any
   borders or scaling.
   
   This control is display only and normally does not permit any further
   interaction.
 */
class PStaticIcon : public PControl
{
  PCLASSINFO(PStaticIcon, PControl);

  public:
    /** Create a static icon control with the icon. */
    PStaticIcon(
      PInteractor * parent    /// Interactor into which the control is placed
    );
    PStaticIcon(
      PInteractor * parent,   /// Interactor into which the control is placed.
      const PIcon & icn       /// Icon to display in the control.
    );

    /** Create control from interactor layout with the specified control ID. */
    PStaticIcon(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      void * valuePtr             /// Unused value pointer for static control.
    );

   /**Destroy the static icon control. If the icon has no more references it
       too will be destroyed.
     */
    virtual ~PStaticIcon();


  /**@name New functions for class */
   /**Set the default (minimum) dimensions for the control. This is determined
       by the size of the icon.
     */
    void DefaultDimensions();

    /** Set the icon to be displayed in the control. */
    void SetIcon(
      const PIcon & icn   /// New icon for control.
    );

   /**Get the current icon used in the control.
    
       @return
       icon object with \Ref{PContainer} reference to the loaded icon.
     */
    PIcon GetIcon() const;


  protected:
    /** Icon used by the static control. */
    PIcon icon;


  private:
    void Construct();
    // Common construction code for the icon control.



// Include platform dependent part of class
#include <pwlib/staticon.h>
};


// End Of File ///////////////////////////////////////////////////////////////
