/*
 * tbutton.h
 *
 * Text button control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: tbutton.h,v $
 * Revision 1.17  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.16  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.15  1999/03/09 08:01:50  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.14  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.13  1998/09/23 06:29:25  robertj
 * Added open source copyright license.
 *
 * Revision 1.12  1995/06/17 11:13:30  robertj
 * Documentation update.
 *
 * Revision 1.11  1995/03/14 12:42:45  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.10  1994/12/21  11:54:40  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.9  1994/10/30  11:47:24  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.8  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.7  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.6  1994/04/03  08:34:18  robertj
 * Added help and focus functionality.
 *
 * Revision 1.5  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.4  1994/01/15  02:50:29  robertj
 * Moved control contructors to common and added a platform dependent Construct() function.
 *
 * Revision 1.3  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.2  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.1  1993/08/24  00:27:59  robertj
 * Initial revision
 *
 */


#define _PTEXTBUTTON

#ifdef __GNUC__
#pragma interface
#endif


class PTextButton : public PPushButton
{
  PCLASSINFO(PTextButton, PPushButton)
/* A simple push button control with text in it, which is used to initiate an
   action.

   The exact appearence of an instance of a PTextButton is platform dependent.
   The implementation may not be completely orthoganal to other descendents of
   PPushButton. This is to yield maximum caompatibility with the native
   platforms button controls. For example, in MS-Windows the
   \Ref{PPushButton::DrawButtonSurround()} and
   \Ref{PPushButton::OnDrawFace()} functions are never actually called so the
   correct button is drawn. Thus if in a later version of the operating system
   the standard push buttons visual appearance is changed, PWLib applications
   will chane with it. Other button types such as \Ref{PImageButtons} will
   remain as they were.
   
   A consequence of this is that the application should never descend from this
   class if the appearance is to be changed. Descend directly from PPushButton
   and draw the text contents. This could simply consist of the line:

        #canvas.DrawString(rect, GetName(), PCanvas::Centred);#
 */

  public:
    /** Create a push button control with the specified attributes. */
    PTextButton(
      PInteractor * parent,    /// Interactor into which the control is placed.
      ButtonType deflt = OrdinaryButton  /// Flag for is the default button.
    );
    PTextButton(
      PInteractor * parent,    /// Interactor into which the control is placed.
      const PString & name,    /// Name to be placed next in the button.
      ButtonType deflt = OrdinaryButton  /// Flag for is the default button.
    );
    PTextButton(
      PInteractor * parent,    /// Interactor into which the control is placed.
      const PString & name,    /// Name to be placed next in the button.
      const PNotifier & notify,  /// Function to call when changes state.
      ButtonType deflt = OrdinaryButton  /// Flag for is the default button.
    );

   /**Create control from interactor layout with the specified control ID.
       This is mainly used in support of resource based dialogs.
     */
    PTextButton(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      void * valuePtr             /// Variable to change to the button state.
    );

    /** Destroy the text push button control. */
    virtual ~PTextButton();


  /**@name New functions for class */
    /** Set the default (minimum) dimensions for the control */
    void DefaultDimensions();


    /**@name System callback functions. */
   /**Draw the contents (face) of the pushbutton.

       On some platforms this may be a dummy function which does nothing and
       is never called.
     */
    virtual void OnDrawFace(
      PCanvas & canvas,   /// Canvas into which to draw the button face.
      const PRect & rect, /// Bounds in which to draw the button face.
      BOOL hasFocus,      /// Flag indicating that the button has the focus.
      BOOL isSelected     /// Flag indicating the button is currently down.
    );


// Include platform dependent part of class
#include <pwlib/tbutton.h>
};


// End Of File ///////////////////////////////////////////////////////////////
